/**
 * Created by panzhichao on 16/8/2.
 */
'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var net = require('net');
var url = require('url');
var zookeeper = require('node-zookeeper-client');
var qs = require('querystring');
var reg = require('./register');
var decode = require('./decode');
var Encode = require('./encode').Encode;

require('./utils');

var SERVICE_LENGTH = 0;
var COUNT = 0;

/**
 * @param {Object} opt {conn:'zk.dev.pajkdc.com:2181',
 * dubbo:{version:PZC,
 *        dversion:2.3.4.6,
 *        group:'xxx'},
 * dependencies:{}}
 * @constructor
 */

var NZD = function NZD(opt) {
  var self = this;
  this.dubboVer = opt.dubboVer;
  this.application = opt.application;
  this._root = opt.root || 'dubbo';

  this.dependencies = opt.dependencies || {};
  SERVICE_LENGTH = (0, _keys2.default)(this.dependencies).length;
  this.client = zookeeper.createClient(opt.register, {
    sessionTimeout: 30000,
    spinDelay: 1000,
    retries: 5
  });

  this.client.connect();
  this.client.once('connected', function () {
    self._applyServices();
    self._consumer();
  });
};
NZD.prototype._consumer = reg.consumer;

NZD.prototype._applyServices = function () {
  var refs = this.dependencies;
  var self = this;

  for (var key in refs) {
    NZD.prototype[key] = new Service(self.client, self.dubboVer, refs[key], this._root);
  }
};

var Service = function Service(zk, dubboVer, depend, root) {
  this._zk = zk;
  this._hosts = [];
  this._version = depend.version;
  this._group = depend.group;
  this._interface = depend.interface;
  this._root = root;

  this._encodeParam = {
    _dver: dubboVer || '2.5.3.6',
    _interface: depend.interface,
    _version: depend.version,
    _group: depend.group,
    _timeout: depend.timeout || 6000
  };

  this._find(depend.interface);
};

Service.prototype._find = function (path, cb) {
  var self = this;
  self._hosts = [];
  this._zk.getChildren('/' + this._root + '/' + path + '/providers', watch, handleResult);

  function watch(event) {
    self._find(path);
  }

  function handleResult(err, children) {
    var zoo = void 0;
    if (err) {
      if (err.code === -4) {
        console.log(err);
      }
      return console.log(err);
    }
    if (children && !children.length) {
      return console.log('can\'t find  the zoo: ' + path + ' ,pls check dubbo service!');
    }

    for (var i = 0, l = children.length; i < l; i++) {
      zoo = qs.parse(decodeURIComponent(children[i]));
      if (zoo.version === self._version && zoo.group === self._group) {
        self._hosts.push(url.parse((0, _keys2.default)(zoo)[0]).host);
        var methods = zoo.methods.split(',');
        for (var _i = 0, _l = methods.length; _i < _l; _i++) {
          self[methods[_i]] = function (method) {
            return function () {
              return self._execute(method, (0, _from2.default)(arguments));
            };
          }(methods[_i]);
        }
      }
    }
    if (typeof cb === 'function') {
      return cb();
    }
    if (++COUNT === SERVICE_LENGTH) {
      console.log('\x1b[32m%s\x1b[0m', 'Dubbo service init done');
    }
  }
};

Service.prototype._flush = function (cb) {
  this._find(this._interface, cb);
};

Service.prototype._execute = function (method, args) {
  var self = this;
  this._encodeParam._method = method;
  this._encodeParam._args = args;
  var buffer = new Encode(this._encodeParam);

  return new _promise2.default(function (resolve, reject) {
    var client = new net.Socket();
    var host = self._hosts[Math.random() * self._hosts.length | 0].split(':');
    var chunks = [];
    var heap = void 0;
    var bl = 16;
    client.connect(host[1], host[0], function () {
      client.write(buffer);
    });

    client.on('error', function (err) {
      self._flush(function () {
        host = self._hosts[Math.random() * self._hosts.length | 0].split(':');
        client.connect(host[1], host[0], function () {
          client.write(buffer);
        });
      });
    });

    client.on('data', function (chunk) {
      if (!chunks.length) {
        var arr = Array.prototype.slice.call(chunk.slice(0, 16));
        var i = 0;
        while (i < 3) {
          bl += arr.pop() * Math.pow(256, i++);
        }
      }
      chunks.push(chunk);
      heap = Buffer.concat(chunks);
      heap.length >= bl && client.destroy();
    });

    client.on('close', function (err) {
      if (!err) {

        decode(heap, function (err, result) {
          if (err) {
            return reject(err);
          }
          return resolve(result);
        });
      }
    });
  });
};

module.exports = NZD;